@@ -1,4 +1,4 @@
-.calc_inits_by_glm <- function(processed, X.pv=processed$AUGI0_ZX$X.pv, family=processed$family,
+.calc_inits_by_xLM <- function(processed, X.pv=processed$AUGI0_ZX$X.pv, family=processed$family,
                                y=processed$y, ## requested by the formula
                                Y=.get_from_terms_info(terms_info=processed$main_terms_info, which="Y"),
                                prior.weights=processed$prior.weights,   ## do not try to eval() it outside of the .wfit function call; else nasty crashes may occur.
@@ -7,16 +7,16 @@
   ## Prior to optimization the family parameters may not be assigned, so: 
   orifamfam <- family$family
   if (orifamfam %in% c("negbin1","negbin2")) {
-    if (variable_fampar <- inherits(substitute(shape, env=environment(family$aic)),"call")) family <- Poisson(family$link, trunc=environment(family$aic)$trunc) 
+    if (variable_rdispar <- inherits(substitute(shape, env=environment(family$aic)),"call")) family <- Poisson(family$link, trunc=environment(family$aic)$trunc) 
   } else if (orifamfam=="beta_resp") {
-    if (variable_fampar <- inherits(substitute(prec, env=environment(family$aic)),"call")) { # only if beta_prec not assigned
+    if (variable_rdispar <- inherits(substitute(prec, env=environment(family$aic)),"call")) { # only if beta_prec not assigned
       loc_link <- family$link
       family <- beta_resp(link=loc_link, prec=1) 
     }
   } else if (orifamfam=="COMPoisson") {
     loc_link <- family$link
     if (loc_link=="loglambda") loc_link <- "log"
-    if (variable_fampar <- inherits(substitute(nu, env=environment(family$aic)),"call")) family <- poisson(loc_link) # only if COMP_nu not assigned
+    if (variable_rdispar <- inherits(substitute(nu, env=environment(family$aic)),"call")) family <- poisson(loc_link) # only if COMP_nu not assigned
     # problem with test glmmTMB: most of the time is spent on inits for glm when nu<1
     # if (inherits(nuchk <- substitute(nu, env=environment(family$aic)),"call") || nuchk >0.25) family <- poisson(loc_link)
     # : This helps for he GLM but not much otherwise. Other ideas ? __F I X M E__
@@ -57,27 +57,32 @@
     
     resglm <- llm.fit(x=X.pv,  y=drop(Y),  weights = eval(prior.weights),  offset = off, 
                       family = family,  control = processed[["control.glm"]])
-    # if ( ( ! .spaMM.data$options$spaMM_glm_conv_silent)
-    #      && (conv_crit <- environment(spaMM_glm.fit)$spaMM_glm_conv_crit$max>0) # note the read and write into that other function's defining envir...
-    # ) {
-    #   resu$conv_info <- paste(".calc_inits_by_glm() -> spaMM_glm.fit or llm.fit did not yet converge at iteration",
-    #                           resglm$iter,"(criterion:",
-    #                           paste(names(conv_crit),"=",signif(unlist(conv_crit),3),collapse=", "),").\n",
-    #                           "Use <fitting function>(., control.glm=list(maxit=<larger value>,..)) to control this.")
-    #   assign("spaMM_glm_conv_crit",list(max=-Inf) , envir=environment(spaMM_glm.fit)) # So that no-convergence in later call (dispGammaGLM) can be independently assessed
-    # } 
-    ## => ___F I X M E___ need to comment the obscure code for GLMs, used as template, first...
-    overdisp <- as.numeric(deviance(resglm)/df.residual(resglm)) # that's the way the dispersion param is estimated in GLMs
-    # ... since resglm is not a glm nor an HLfit object, this uses the stats:::<>.default method of the two extractors ...
-    if (locfamfam=="beta_resp") {
-      resu$lambda <- min(1,overdisp) # think of binary logistic model... but # (___F I X M E__)_ still a bit large  (0.2767528) in the 1st beta_resp example. Quick patch:
-      if (variable_fampar) resu$beta_prec <- min(10, max(0.15, 1/(2*overdisp)))
-    } else resu$lambda <- overdisp
+    if ( ! resglm$converged && family$link !="inverse") {
+      ## divergence more likely than from spaMM_glm.fit, and fewer options then...  a simple control of 'start' may work well for log link:
+      resglm <- llm.fit(x=X.pv,  y=drop(Y),  weights = eval(prior.weights),  offset = off, 
+                        start = rep(0,ncol(X.pv)),
+                        family = family,  control = processed[["control.glm"]])
+    }
+    if ( (! resglm$converged ) && ( ! .spaMM.data$options$xLM_conv_silent)
+         && (conv_crit <- .spaMM.data$options$xLM_conv_crit$max>0)) { 
+      resu$conv_info <- paste("In initialization, llm.fit() did not converge within",
+                              resglm$iter,"iterations (criterion:",
+                              paste(names(conv_crit),"=",signif(unlist(conv_crit),3),collapse=", "),").\n",
+                              "Use <fitting function>(., control.glm=list(maxit=<larger value>,..)) to control this.")
+      # No dispGammaGLM for residual variation in these families => no need to reset .spaMM.data$options$xLM_conv_crit here ? (see explanation in glm case)
+      # BUT: cross effects in mv fits ? => play safe
+      .spaMM.data$options$xLM_conv_crit <- list(max=-Inf) # So that no-convergence in later .calc_dispGammaGLM) call can be independently assessed
+    }
+    #
+    overdisp <- as.numeric(deviance(resglm)/df.residual(resglm)) # that's the way the residual dispersion param is estimated in GLMs;
+    # if (overdisp<1e-10) browser()
+    # since resglm is not a glm nor an HLfit object, this uses the stats:::<>.default method of the two extractors.
+    if (locfamfam=="beta_resp") { # Now, 'overdisp is 1 if lambda=0' We need lambda low in that case, but not too low (optim issues)
+      resu$lambda <- max(0.1,min(log(overdisp),1)) #  1st beta_resp example of the doc is good test. 
+      if (variable_rdispar) resu$beta_prec <- min(10, max(0.15, 1/(2*overdisp))) # will be ignored by rdisPars controls for *structured* beta_prec
+    } else resu$lambda <- max(0.1,log(overdisp))
     
-    ## ____F I X M E___ fact is that init may diverge (more likely than from spaMM_glm.fit) and I have no alternative...
-    # Depending on $converged I could use overdisp or something else ?
-    
-    ## I tried weird things here in version 4.0.14
+    ## I tried weird things here in version 4.0.14. In any case don't forget there is also a distinct .calc_fam_corrected_guess() function.
 
   } else { ## LOCfamily is GLM, even when fitted by obsInfo
     # (1) This handles truncated fams (since glm -> glm.fit handles Tpoisson() etc)
@@ -89,21 +94,32 @@
                                     offset = off, family = family, 
                                     control = processed[["control.glm"]]))
     if (inherits((resglm <- tryglm$value),"error") || 
-        ( ! resglm$converged && any(fitted(resglm)>1e20)) # this occurred in Gamma(log) models or negbin(log)
-    ) {
-      # if (locfamfam=="gaussian" && family$link %in% c("log","inverse")) family$initialize <- .gauss_initialize_in_Xbeta_image
-      resglm <- spaMM_glm.fit(x=X.pv, # ->LevM -> COMP bottleneck
+        ( ! resglm$converged ### && any(fitted(resglm)>1e20)
+          ) # this occurred in Gamma(log) models or negbin(log)
+    ) { # not often... hzut in test-COMPoisson-difficult (glm not converged).
+      # use more robust method, and for log link, additionally set an initial value that will override the default mustart <- y of family()$initialize..
+      if (family$link=="log") {
+        start <- rep(0, ncol(X.pv))
+      } else start <- NULL
+      resglm <- spaMM_glm.fit(x=X.pv, # -> LevM -> COMP bottleneck
                               y=Y, 
                               weights = eval(prior.weights), 
-                              offset = off, family = family, 
+                              offset = off, family = family, start=start,
                               control = processed[["control.glm"]])
-      if ( ( ! .spaMM.data$options$spaMM_glm_conv_silent)
-           && (conv_crit <- environment(spaMM_glm.fit)$spaMM_glm_conv_crit$max>0)) {
-        resu$conv_info <- paste(".calc_inits_by_glm() -> spaMM_glm.fit or llm.fit did not yet converge at iteration",
-                                resglm$iter,"(criterion:",
+      if ( ( ! resglm$converged) && ( ! .spaMM.data$options$xLM_conv_silent)
+           && (conv_crit <- .spaMM.data$options$xLM_conv_crit$max>0)) {
+        resu$conv_info <- paste("In initialization, spaMM_glm.fit() did not converge within",
+                                resglm$iter,"iterations (criterion:",
                                 paste(names(conv_crit),"=",signif(unlist(conv_crit),3),collapse=", "),").\n",
                                 "Use <fitting function>(., control.glm=list(maxit=<larger value>,..)) to control this.")
-        assign("spaMM_glm_conv_crit",list(max=-Inf) , envir=environment(spaMM_glm.fit)) # So that no-convergence in later call (dispGammaGLM) can be independently assessed
+        # This info will then be copied as part of the whole 'inits_by_xLM' into processed$envir$inits_by_xLM;
+        #  a later .calc_inits_by_xLM() call may overwrite the whole 'inits_by_xLM' so that only the last initialization's $conv_info can be reported,
+        #  by a message in a fit summary.
+        # .calc_dispGammaGLM() has a distinct tracking mechanism: a *warning* may be emitted each time .check_conv_dispGammaGLM_reinit() is called,
+        #  that is, within each call of the main fitting functions, incl. repetitive 'inner' calls of (say) HLfit() within fitme(). 
+        #  Nevertheless, the .calc_dispGammaGLM() tracking mechanism also reads convergence info in .spaMM.data$options$xLM_conv_crit 
+        #  since this is where xLM fitting functions write it. Hence the need to reset it:
+        .spaMM.data$options$xLM_conv_crit <- list(max=-Inf) # So that no-convergence in later .calc_dispGammaGLM) call can be independently assessed
       }
     } else if ( ! is.null(tryglm$warning) && 
                 substring(tryglm$warning$message,0,14)=="maxn truncated") .spaMM.data$options$COMP_maxn_warned <- FALSE
@@ -147,10 +163,10 @@
 
 # function called within HLfit for missing inits... 
 # but also sometimes in fitme_body prior to optimization (cf complex condition for call of .eval_init_lambda_guess())
-.get_inits_by_glm <- function(processed, reset=FALSE,
+.get_inits_by_xLM <- function(processed, reset=FALSE,
                               # args used only as promises:
                               X.pv=processed$AUGI0_ZX$X.pv, family=processed$family) {
-  if ( is.null(inits_by_glm <- processed$envir$inits_by_glm)) {
+  if ( is.null(inits_by_xLM <- processed$envir$inits_by_xLM)) {
     if ( ! is.null(vec_nobs <- processed$vec_nobs) ) {
       do_eval <- rep(TRUE, length(vec_nobs))
       new_mvlist <- new_phi_ests <- vector("list", length(vec_nobs))
@@ -163,10 +179,10 @@
       family <- processed$families[[mv_it]] # necessary bc reset=quote(...) uses it!
       do_eval[mv_it] <- eval(reset)
     }
-    new_mvlist <- inits_by_glm$mvlist
-    new_phi_ests <- inits_by_glm$phi_ests 
-    beta_eta <- inits_by_glm$beta_eta
-    lambdas <- inits_by_glm$lambdas
+    new_mvlist <- inits_by_xLM$mvlist
+    new_phi_ests <- inits_by_xLM$phi_ests 
+    beta_eta <- inits_by_xLM$beta_eta
+    lambdas <- inits_by_xLM$lambdas
   } else {
     family <- processed$family
     do_eval <- eval(reset)
@@ -175,11 +191,11 @@
   if (any(do_eval)) {
     if ( ! is.null(vec_nobs)) {
       cum_nobs <- attr(processed$families,"cum_nobs")
-      cum_ncol_X <- attr(X.pv,"cum_ncol")
+      col_ranges <-  attr(X.pv,"col_ranges")
       for (mv_it in which(do_eval)) {
         resp_range <- .subrange(cumul=cum_nobs, it=mv_it)
-        col_range <- cum_ncol_X[mv_it]+seq_len(cum_ncol_X[mv_it+1L]-cum_ncol_X[mv_it]) # avoid (n+1:n) problem 
-        new_mvlist[[mv_it]] <- .calc_inits_by_glm(processed, X.pv=X.pv[resp_range,col_range, drop=FALSE],                                                   family=processed$families[[mv_it]],
+        col_range <-  col_ranges[[mv_it]]
+        new_mvlist[[mv_it]] <- .calc_inits_by_xLM(processed, X.pv=X.pv[resp_range,col_range, drop=FALSE],                                                   family=processed$families[[mv_it]],
                                                   y=processed$y[resp_range], ## requested by the formula
                                                   Y=.get_from_terms_info(terms_info=processed$main_terms_info, which="Y", mv_it=mv_it), 
                                                   prior.weights=processed$prior.weights[[mv_it]], ## do not try to eval() it outside of the .wfit function call; else nasty crashes may occur.
@@ -191,9 +207,9 @@
         } else new_phi_ests[[mv_it]] <- 1 # no impact anyway?
       }
       names(new_mvlist) <- names(new_phi_ests) <- which(do_eval)
-      processed$envir$inits_by_glm <- list(mvlist=.modify_list(inits_by_glm$mvlist,new_mvlist), # info. But maybe we won't need it ?
+      processed$envir$inits_by_xLM <- list(mvlist=.modify_list(inits_by_xLM$mvlist,new_mvlist), # info. But maybe we won't need it ?
                                            beta_eta=beta_eta, # vector
-                                           phi_est=.modify_list(inits_by_glm$phi_est,new_phi_ests), # list
+                                           phi_est=.modify_list(inits_by_xLM$phi_est,new_phi_ests), # list
                                            lambdas=lambdas, # vector
                                            lambda=exp(mean(log(lambdas))) # scalar
       )
@@ -201,9 +217,9 @@
       if (! is.null(processed$X_off_fn)) {
         X.pv <- environment(processed$X_off_fn)$X_off
       } else X.pv <- processed$AUGI0_ZX$X.pv
-      processed$envir$inits_by_glm <- .calc_inits_by_glm(processed, X.pv=X.pv) # univariate response case
+      processed$envir$inits_by_xLM <- .calc_inits_by_xLM(processed, X.pv=X.pv) # univariate response case
     }
   }
-  return(processed$envir$inits_by_glm)
+  return(processed$envir$inits_by_xLM)
 } 
 
